@@ -13,17 +13,17 @@ use egui::{paint::Mesh, Rect};
 use vulkano::{
     buffer::{BufferAccess, BufferUsage, CpuAccessibleBuffer},
     command_buffer::{
-        AutoCommandBufferBuilder, CommandBufferUsage, DynamicState, PrimaryAutoCommandBuffer,
+        AutoCommandBufferBuilder, CommandBufferUsage, PrimaryAutoCommandBuffer,
         SecondaryAutoCommandBuffer, SubpassContents,
     },
-    descriptor_set::{DescriptorSet, PersistentDescriptorSet, layout::DescriptorSetLayout},
+    descriptor_set::{layout::DescriptorSetLayout, DescriptorSet, PersistentDescriptorSet},
     device::{Device, Queue},
-    format::Format,
+    format::{ClearValue, Format},
     image::{view::ImageView, AttachmentImage, ImageViewAbstract},
     pipeline::{
         blend::{AttachmentBlend, BlendFactor},
         viewport::{Scissor, Viewport},
-        GraphicsPipeline, GraphicsPipelineAbstract,
+        GraphicsPipeline, PipelineBindPoint,
     },
     render_pass::{Framebuffer, RenderPass, Subpass},
     sampler::{Filter, MipmapMode, Sampler, SamplerAddressMode},
@@ -49,12 +49,13 @@ vulkano::impl_vertex!(EguiVertex, position, tex_coords, color);
 pub struct Renderer {
     gfx_queue: Arc<Queue>,
     render_pass: Option<Arc<RenderPass>>,
+    is_overlay: bool,
 
     format: vulkano::format::Format,
 
     vertex_buffer: Arc<CpuAccessibleBuffer<[EguiVertex]>>,
     index_buffer: Arc<CpuAccessibleBuffer<[u32]>>,
-    pipeline: Arc<dyn GraphicsPipelineAbstract + Send + Sync>,
+    pipeline: Arc<GraphicsPipeline>,
 
     egui_texture_version: u64,
     egui_texture_desc_set: Arc<dyn DescriptorSet + Send + Sync>,
@@ -87,15 +88,36 @@ impl Renderer {
             egui_texture_version: 0,
             egui_texture_desc_set: font_desc_set,
             user_texture_desc_sets: vec![],
+            is_overlay: false,
         }
     }
 
-    /// Creates a new [EguiVulkanoRenderer] which is responsible for rendering egui with its own renderpass
+    /// Creates a new [Renderer] which is responsible for rendering egui with its own renderpass
     /// See examples
-    pub fn new_with_render_pass(gfx_queue: Arc<Queue>, final_output_format: Format) -> Renderer {
+    pub fn new_with_render_pass(
+        gfx_queue: Arc<Queue>,
+        final_output_format: Format,
+        is_overlay: bool,
+    ) -> Renderer {
         // Create Gui render pass with just depth and final color
-        let render_pass = Arc::new(
-            vulkano::ordered_passes_renderpass!(gfx_queue.device().clone(),
+        let render_pass = Arc::new(if is_overlay {
+            vulkano::single_pass_renderpass!(gfx_queue.device().clone(),
+                attachments: {
+                    final_color: {
+                        load: Load,
+                        store: Store,
+                        format: final_output_format,
+                        samples: 1,
+                    }
+                },
+                pass: {
+                        color: [final_color],
+                        depth_stencil: {}
+                }
+            )
+            .unwrap()
+        } else {
+            vulkano::single_pass_renderpass!(gfx_queue.device().clone(),
                 attachments: {
                     final_color: {
                         load: Clear,
@@ -104,16 +126,13 @@ impl Renderer {
                         samples: 1,
                     }
                 },
-                passes: [
-                    {
+                pass: {
                         color: [final_color],
-                        depth_stencil: {},
-                        input: []
-                    }
-                ]
+                        depth_stencil: {}
+                }
             )
-            .unwrap(),
-        );
+            .unwrap()
+        });
 
         let (vertex_buffer, index_buffer) = Self::create_buffers(gfx_queue.device().clone());
 
@@ -139,6 +158,7 @@ impl Renderer {
             egui_texture_version: 0,
             egui_texture_desc_set: font_desc_set,
             user_texture_desc_sets: vec![],
+            is_overlay,
         }
     }
 
@@ -171,10 +191,7 @@ impl Renderer {
         (vertex_buffer, index_buffer)
     }
 
-    fn create_pipeline(
-        gfx_queue: Arc<Queue>,
-        subpass: Subpass,
-    ) -> Arc<dyn GraphicsPipelineAbstract + Send + Sync> {
+    fn create_pipeline(gfx_queue: Arc<Queue>, subpass: Subpass) -> Arc<GraphicsPipeline> {
         let vs =
             vs::Shader::load(gfx_queue.device().clone()).expect("failed to create shader module");
         let fs =
@@ -218,13 +235,10 @@ impl Renderer {
             0.0,
         )
         .expect("Failed to create sampler");
-        Arc::new(
-            PersistentDescriptorSet::start(layout.clone())
-                .add_sampled_image(image.clone(), sampler)
-                .unwrap()
-                .build()
-                .expect("Failed to create descriptor set with sampler"),
-        )
+        let mut builder = PersistentDescriptorSet::start(layout.clone());
+        builder.add_sampled_image(image.clone(), sampler).unwrap();
+        let set = builder.build().unwrap();
+        Arc::new(set)
     }
 
     /// Registers a user texture. User texture needs to be unregistered when it is no longer needed
@@ -305,7 +319,7 @@ impl Renderer {
             y: max.y.clamp(min.y, framebuffer_dimensions[1] as f32),
         };
         Scissor {
-            origin: [min.x.round() as i32, min.y.round() as i32],
+            origin: [min.x.round() as u32, min.y.round() as u32],
             dimensions: [(max.x.round() - min.x) as u32, (max.y.round() - min.y) as u32],
         }
     }
@@ -386,7 +400,6 @@ impl Renderer {
     fn start<I>(
         &mut self,
         final_image: I,
-        clear_color: [f32; 4],
     ) -> (AutoCommandBufferBuilder<PrimaryAutoCommandBuffer>, [u32; 2])
     where
         I: ImageViewAbstract + Clone + Send + Sync + 'static,
@@ -418,7 +431,7 @@ impl Renderer {
         // Add clear values here for attachments and begin render pass
         command_buffer_builder
             .begin_render_pass(framebuffer, SubpassContents::SecondaryCommandBuffers, vec![
-                clear_color.into(),
+                if !self.is_overlay { [0.0; 4].into() } else { ClearValue::None },
             ])
             .unwrap();
         (command_buffer_builder, img_dims)
@@ -431,14 +444,12 @@ impl Renderer {
         clipped_meshes: Vec<egui::ClippedMesh>,
         before_future: F,
         final_image: I,
-        clear_color: [f32; 4],
     ) -> Box<dyn GpuFuture>
     where
         F: GpuFuture + 'static,
         I: ImageViewAbstract + Clone + Send + Sync + 'static,
     {
-        let (mut command_buffer_builder, framebuffer_dimensions) =
-            self.start(final_image, clear_color);
+        let (mut command_buffer_builder, framebuffer_dimensions) = self.start(final_image);
         egui_context.update_elapsed_time();
         self.update_font_texture(egui_context);
 
@@ -514,18 +525,6 @@ impl Renderer {
             }
 
             let scissors = vec![self.get_rect_scissor(egui_context, framebuffer_dimensions, rect)];
-            let dynamic_state = DynamicState {
-                viewports: Some(vec![Viewport {
-                    origin: [0.0, 0.0],
-                    dimensions: [
-                        framebuffer_dimensions[0] as f32,
-                        framebuffer_dimensions[1] as f32,
-                    ],
-                    depth_range: 0.0..1.0,
-                }]),
-                scissors: Some(scissors),
-                ..DynamicState::none()
-            };
             let vertices_count = mesh.vertices.len() as DeviceSize;
             let indices_count = mesh.indices.len() as DeviceSize;
             // Resize buffers if needed
@@ -559,14 +558,26 @@ impl Renderer {
                 self.egui_texture_desc_set.clone()
             };
             builder
-                .draw_indexed(
-                    self.pipeline.clone(),
-                    &dynamic_state,
-                    vec![vertices.clone()],
-                    indices.clone(),
+                .bind_pipeline_graphics(self.pipeline.clone())
+                .set_viewport(0, vec![Viewport {
+                    origin: [0.0, 0.0],
+                    dimensions: [
+                        framebuffer_dimensions[0] as f32,
+                        framebuffer_dimensions[1] as f32,
+                    ],
+                    depth_range: 0.0..1.0,
+                }])
+                .set_scissor(0, scissors)
+                .bind_descriptor_sets(
+                    PipelineBindPoint::Graphics,
+                    self.pipeline.layout().clone(),
+                    0,
                     desc_set,
-                    push_constants,
                 )
+                .push_constants(self.pipeline.layout().clone(), 0, push_constants)
+                .bind_vertex_buffers(0, vertices.clone())
+                .bind_index_buffer(indices.clone())
+                .draw_indexed(indices.len() as u32, 1, 0, 0, 0)
                 .unwrap();
             vertex_start += vertices_count;
             index_start += indices_count;
